from django.conf import settings
from django.utils._os import safe_join
from django.http.response import Http404
from django.shortcuts import render
from django.template.base import TemplateDoesNotExist

from template_pages.utils import get_ctx


def routing_view(request, path):
    # check if path is plain text not utf-8
    try:
        path = str(path)
    except UnicodeEncodeError:
        raise Http404

    # try safely joining paths
    try:
        path = safe_join('/', path)
    except ValueError:
        # in case of trying to get outside of
        # /template_pages/
        raise Http404

    # get extra context for current path
    ctx = {}
    if getattr(settings, 'TEMPLATE_PAGES_CONTEXT_MODULE', False):
        ctx = get_ctx(request, path)

    # try template_pages/#path.html
    first_template_path = u'template_pages%s.html' % path
    try:
        return render(request, first_template_path, ctx)
    except TemplateDoesNotExist, e:
        # Some tags (eg. include) raise
        # TemplateDoesNotExists(PATH) so
        # we need to check template path
        # with raised error and raise them
        if str(e) != first_template_path:
            raise

        # try template_pages/#path/index.html
        try:
            return render(request, u'template_pages%s/index.html' % path, ctx)
        except TemplateDoesNotExist:
            # Return TemplateDoesNotExist
            # errors in debug mode (for
            # developers) otherwise page
            # doesn't exists...
            if settings.DEBUG:
                raise
            raise Http404




def _hello_world_context():
    return {
        'context' : 'Hello World!',
    }

def index(request):
    return _hello_world_context()

def test1(request):
    return _hello_world_context()

def test2(request):
    return _hello_world_context()

def test2_test3(request):
    return _hello_world_context()



from django.conf.urls import patterns, url

urlpatterns = patterns('template_pages.views',
    url(r'^(?P<path>[^.\n]*)/$', 'routing_view', name='template_pages_routing_view'),
    url(r'^$', 'routing_view', {'path':''}, name='template_pages_routing_view'),
)



# -*- coding: utf-8 -*-
from os.path import dirname, join
from django.test import TestCase, Client
from django.test.utils import override_settings
from django.template.base import TemplateDoesNotExist
from django.http.response import Http404

_path = dirname(__file__)

@override_settings(
    TEMPLATE_DIRS=(join(_path, 'templates'),),
    INSTALLED_APPS=('template_pages'),
    MIDDLEWARE_CLASSES=(
        'django.middleware.common.CommonMiddleware',
        'django.contrib.sessions.middleware.SessionMiddleware',
        'django.middleware.csrf.CsrfViewMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'django.contrib.messages.middleware.MessageMiddleware',
    ),
    TEMPLATE_PAGES_CONTEXT_MODULE=None,
)
class TestRoutingView(TestCase):
    urls = 'template_pages.tests.urls'
    _template_pages_test_context_module = 'template_pages.tests.template_pages_context'

    def setUp(self):
        # Every test needs a client.
        self.client = Client()

    def test_file(self):
        response = self.client.get('/test1/')
        self.assertContains(response, 'This is test1.')

    @override_settings(
        TEMPLATE_PAGES_CONTEXT_MODULE=_template_pages_test_context_module
    )
    def test_file_with_context(self):
        response = self.client.get('/test1/')
        self.assertContains(response, 'Hello World!')

    def test_index(self):
        response = self.client.get('/')
        self.assertContains(response, 'This is index.')

    @override_settings(
        TEMPLATE_PAGES_CONTEXT_MODULE=_template_pages_test_context_module
    )
    def test_index_with_context(self):
        response = self.client.get('/')
        self.assertContains(response, 'Hello World!')

    def test_directory_index(self):
        response = self.client.get('/test2/')
        self.assertContains(response, 'This is test2 index.')

    @override_settings(
        TEMPLATE_PAGES_CONTEXT_MODULE=_template_pages_test_context_module
    )
    def test_directory_index_with_context(self):
        response = self.client.get('/test2/')
        self.assertContains(response, 'Hello World!')

    def test_file_inside_directory(self):
        response = self.client.get('/test2/test3/')
        self.assertContains(response, 'This is test2/test3.')

    @override_settings(
        TEMPLATE_PAGES_CONTEXT_MODULE=_template_pages_test_context_module
    )
    def test_file_inside_with_context(self):
        response = self.client.get('/test2/test3/')
        self.assertContains(response, 'Hello World!')

    def test_unicode_path(self):
        response = self.client.get(u'/django-wymiąta/'.encode('utf-8'))
        self.assertEqual(response.status_code, 404)

    def test_dir_up(self):
        response = self.client.get('/../index/')
        self.assertEqual(response.status_code, 404)

    def test_same_dir(self):
        response = self.client.get('/./test2/')
        self.assertEqual(response.status_code, 404)

    def test_redirect_wo_trailing_slash(self):
        response = self.client.get('/test2')
        self.assertRedirects(response, '/test2/', status_code=301)

    def test_not_supressing_errors(self):
        self.assertRaises(TemplateDoesNotExist, self.client.get, '/test4/')

    @override_settings(
        DEBUG=True
    )
    def test_not_supressing_errors_in_debug(self):
        self.assertRaises(TemplateDoesNotExist, self.client.get, '/test-rocket-cat-does-not-exists/')

    @override_settings(
        DEBUG=True
    )
    def test_supressing_errors__not_in_debug(self):
        response = self.client.get('/test-rocket-cat-does-not-exists/')
        self.assertEqual(response.status_code, 404)




#!/usr/bin/env python

from distutils.core import setup

setup(name='django-template-pages',
    version='1.0.1',
    description='Flat pages based on template directory structure for django.',
    author='Leszek Piatek',
    author_email='lpiatek@gmail.com',
    url='https://github.com/iRynek/django-template-pages/',
    license='GPL',
    packages=[
        'template_pages',
        'template_pages.tests',
    ],
    requires=[
        'Django',
    ],
)



"""
Utilities module used in template_pages.
"""
from django.conf import settings


def get_ctx(request, path):
    """
    Get context from given path using
    settings.TEMPLATE_PAGES_CONTEXT_MODULE
    and 'this_is_pathto' function for this/is/path-to path.
    (/ is _ and - are ommited)
    
    When path is not set we search for 'index' function in
    settings.TEMPLATE_PAGES_CONTEXT_MODULE.
    
    We always update RequestContext dictionary
    """
    ctx = {}
    if getattr(settings, 'TEMPLATE_PAGES_CONTEXT_MODULE', False):
        # get rid of / at beginning
        if path[0] == '/':
            path = path[1:]

        if not path:
            function_name = 'index'
        else:
            function_name = path.replace(u'/', u'_').replace(u'-', u'')

        module = __import__(settings.TEMPLATE_PAGES_CONTEXT_MODULE, globals(), locals(), [function_name])
        ctx_function = getattr(module, function_name, None)
        if ctx_function:
            ctx = ctx_function(request)
    return ctx

